// -*- Pike -*-
// original author: mirar
//! @appears Regexp.PCRE

#pike __REAL_VERSION__

inherit .____Regexp_PCRE; // C-module

// there are other maybe useful stuff in _Regexp_PCRE
// so don't stop compiling because it's not complete:
#if constant(.____Regexp_PCRE._pcre) // there are other maybe useful stuff

//! The main regexp class. Will provide anything needed
//! for matching regexps.
//!
//! There are subclasses that adds wrappers for widestrings,
//! and to optimize the regexp pattern.
class Plain
{
   inherit _pcre;

/***************************************************************/

   protected int(0..0) handle_exec_error(int error_no)
   {
      switch (error_no)
      {
	 case ERROR.NOMATCH: 
	    return UNDEFINED; 
	 case ERROR.NOMEMORY:
	    error("out of memory in exec() (ERROR.NOMEMORY)\n");
	 default:
	    error("error returned from exec: %s\n",
		  ([ERROR.NULL   :"ERROR.NULL",
		    ERROR.BADOPTION:"ERROR.BADOPTION",
		    ERROR.BADMAGIC :"ERROR.BADMAGIC",
		    ERROR.UNKNOWN_NODE:"ERROR.UNKNOWN_NODE",
#if constant(.____Regexp_PCRE.ERROR.MATCHLIMIT)
		    ERROR.MATCHLIMIT  :"ERROR.MATCHLIMIT",
#endif /* PCRE.ERROR.MATCHLIMIT */
#if constant(.____Regexp_PCRE.ERROR.CALLOUT)
		    ERROR.CALLOUT     :"ERROR.CALLOUT",
#endif /* PCRE.ERROR.CALLOUT */
		  ])[error_no] || sprintf("unknown error: %d\n",error_no));
      }
   }

//! Matches a subject against the pattern,
//! returns an array where the first element are the whole match,
//! and the subsequent are the matching subpatterns.
//! Returns 0 if there was no match.
//!
//! example:
//! @code
//! > Regexp.PCRE.Plain("i\(.*\) is \(.*\)u")->split2("pike is fun");
//! Result: ({ 
//!              "ike is fu",
//!              "ke",
//!              "f"
//!          })
//! @endcode
   array(string)|int(0..0) split2(string subject,void|int startoffset)
   {
      array(int)|int v=exec(subject,startoffset);
      if (intp(v)) return handle_exec_error([int]v);
      return split_subject(subject,[array(int)]v); 
   }

//! Matches a subject against the pattern,
//! compatible with the old split method:
//! returns an array of the subpatterns,
//! or if there are no subpattern but still a match, ({0}).
//! Returns 0 if there was no match.
//!
//! example:
//! @code
//! > Regexp.PCRE.Plain("i\(.*\) is \(.*\)u")->split("pike is fun");
//! (1) Result: ({
//!                 "ke",
//!                 "f"
//!             })
//! > Regexp.PCRE.Plain("is fun")->split("pike is fun"); 
//! (4) Result: ({
//!                 0
//!             })
//! @endcode
   array(string)|int(0..0) split(string subject,void|int startoffset)
   {
      array(string)|int(0..0) v=split2(subject,startoffset);
      if (intp(v)) return v;
      if (sizeof(v)>1) return v[1..];
      else return copy_value(({0}));
   }

//! returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns true returns t